@@ -468,28 +468,15 @@ static void setWidget(QWidget* parent, QWidget* child)
 // Utility function used to get the name of a control
 // The name of the control indicates actions, toggles ...
 // The action name alias can contain parameters
-// The accessible name property can override controls's name
 static bool translateName(QtWidget& w, String& name, NamedList** params = 0)
 {
-    static String actionProp = "accessibleName";
-
     if (w.invalid())
 	return false;
-    bool noAlias = true;
-    if (w.type() != QtWidget::Action) {
-	noAlias = w->accessibleName().isEmpty();
-	if (noAlias) 
-	    QtClient::getUtf8(name,w->objectName());
-	else
-	    QtClient::getUtf8(name,w->accessibleName());
-    }
-    else {
-	QtClient::getProperty(w.action(),actionProp,name);
-	noAlias = name.null();
-	if (noAlias)
-	    QtClient::getUtf8(name,w.action()->objectName());
-    }
-    if (noAlias)
+    if (w.type() != QtWidget::Action)
+	QtClient::getIdentity(w.widget(),name);
+    else
+	QtClient::getIdentity(w.action(),name);
+    if (!name)
 	return true;
     // Check params
     int pos = name.find('|');
@@ -790,7 +777,6 @@ QtWindow::QtWindow(const char* name, const char* description, const char* alias)
     m_maximized(false), m_mainWindow(false)
 {
     setObjectName(QtClient::setUtf8(m_id));
-    setAccessibleName(QtClient::setUtf8(description));
 }
 
 QtWindow::~QtWindow()
@@ -1999,7 +1985,6 @@ bool QtWindow::eventFilter(QObject* obj, QEvent* event)
 {
     if (!obj)
 	return false;
-#if QT_VERSION >= 0x040200
     // Apply dynamic properties changes
     if (event->type() == QEvent::DynamicPropertyChange) {
 	String name = YQT_OBJECT_NAME(obj);
@@ -2053,7 +2038,6 @@ bool QtWindow::eventFilter(QObject* obj, QEvent* event)
 		prop.c_str(),value.c_str(),name.c_str());
 	return false;
     }
-#endif
     if (event->type() == QEvent::KeyPress) {
 	static int mask = Qt::SHIFT | Qt::CTRL | Qt::ALT;
 
@@ -2235,11 +2219,11 @@ void QtWindow::doInit()
     m_visible = m_mainWindow || m_visible;
 
     // Create custom widgets from
-    // accessibleName=customwidget|[separator=sep|] sep widgetclass sep widgetname [sep param=value]
+    // _yate_identity=customwidget|[separator=sep|] sep widgetclass sep widgetname [sep param=value]
     QList<QFrame*> frm = qFindChildren<QFrame*>(this);
     for (int i = 0; i < frm.size(); i++) {
 	String create;
-	QtClient::getUtf8(create,frm[i]->accessibleName());
+	QtClient::getProperty(frm[i],"_yate_identity",create);
 	if (!create.startSkip("customwidget|",false))
 	    continue;
 	char sep = '|';
@@ -2412,7 +2396,6 @@ void QtWindow::doInit()
 	}
     }
 
-#if QT_VERSION >= 0x040200
     // Install event filer and apply dynamic properties
     QList<QObject*> w = qFindChildren<QObject*>(this);
     for (int i = 0; i < w.size(); i++) {
@@ -2436,7 +2419,6 @@ void QtWindow::doInit()
 	    eventFilter(w[i],&ev);
 	}
     }
-#endif
 
     qRegisterMetaType<QModelIndex>("QModelIndex");
     qRegisterMetaType<QTextCursor>("QTextCursor");
@@ -2518,6 +2500,9 @@ void QtClient::cleanup()
 
 void QtClient::run()
 {
+    const char* style = Engine::config().getValue("client","style");
+    if (style && !QApplication::setStyle(QString::fromUtf8(style)))
+	Debug(ClientDriver::self(),DebugWarn,"Could not set Qt style '%s'",style);
     int argc = 0;
     char* argv =  0;
     m_app = new QApplication(argc,&argv);
@@ -2596,7 +2581,7 @@ bool QtClient::chooseFile(Window* parent, NamedList& params)
     //  child dialog and we might have incomplete (not full) custom styled controls
     QFileDialog* dlg = new QFileDialog(0,setUtf8(params.getValue("caption")),
 	setUtf8(params.getValue("dir")));
-    
+
     if (wnd)
 	dlg->setWindowIcon(wnd->windowIcon());
 
@@ -2888,7 +2873,7 @@ QtEventProxy::QtEventProxy(Type type, QApplication* app)
 		QTimer* timer = new QTimer(this);
 		timer->setObjectName("qtClientIdleTimer");
 		QtClient::connectObjects(timer,SIGNAL(timeout()),this,SLOT(timerTick()));
-		timer->start(1);
+		timer->start(0);
 	    }
 	    break;
 	case AllHidden:
@@ -2906,6 +2891,7 @@ void QtEventProxy::timerTick()
 {
     if (Client::self())
 	Client::self()->idleActions();
+    Thread::idle();
 }
 
 void QtEventProxy::allHidden()
