@@ -118,8 +118,9 @@ bool CallEndpoint::connect(CallEndpoint* peer, const char* reason, bool notify)
     DDebug(DebugAll,"CallEndpoint '%s' connecting peer %p to [%p]",m_id.c_str(),peer,this);
 
 #if 0
-    if (!s_mutex.lock(5000000)) {
-	Debug(DebugFail,"Call connect failed - deadlock on call endpoint mutex!");
+    Lock lock(s_mutex,5000000);
+    if (!lock.locked()) {
+	Debug(DebugFail,"Call connect failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
 	Engine::restart(0);
 	return false;
     }
@@ -147,10 +148,6 @@ bool CallEndpoint::connect(CallEndpoint* peer, const char* reason, bool notify)
     setDisconnect(0);
     connected(reason);
 
-#if 0
-    s_mutex.unlock();
-#endif
-
     return true;
 }
 
@@ -162,7 +159,7 @@ bool CallEndpoint::disconnect(bool final, const char* reason, bool notify, const
 
     Lock lock(s_mutex,5000000);
     if (!lock.locked()) {
-	Debug(DebugFail,"Call disconnect failed - deadlock on call endpoint mutex!");
+	Debug(DebugFail,"Call disconnect failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
 	Engine::restart(0);
 	return false;
     }
@@ -200,6 +197,32 @@ void CallEndpoint::setPeer(CallEndpoint* peer, const char* reason, bool notify,
     }
 }
 
+bool CallEndpoint::getPeerId(String& id) const
+{
+    id.clear();
+    if (!m_peer)
+	return false;
+    Lock lock(s_mutex,5000000);
+    if (!lock.locked()) {
+	Debug(DebugFail,"Peer ID failed - timeout on call endpoint mutex owned by '%s'!",s_mutex.owner());
+	Engine::restart(0);
+	return false;
+    }
+    if (m_peer) {
+	id = m_peer->id();
+	return true;
+    }
+    else
+	return false;
+}
+
+String CallEndpoint::getPeerId() const
+{
+    String id;
+    getPeerId(id);
+    return id;
+}
+
 DataEndpoint* CallEndpoint::getEndpoint(const char* type) const
 {
     if (null(type))
@@ -341,9 +364,6 @@ void Channel::init()
 	    tmp << m_driver->nextid();
 	    setId(tmp);
 	}
-	m_driver->m_total++;
-	m_driver->channels().append(this);
-	m_driver->changed();
 	m_driver->unlock();
     }
     // assign a new billid only to incoming calls
@@ -373,6 +393,23 @@ void Channel::filterDebug(const String& item)
     }
 }
 
+void Channel::initChan()
+{
+    if (!m_driver)
+	return;
+    Lock mylock(m_driver);
+#ifndef NDEBUG
+    if (m_driver->channels().find(this)) {
+	Debug(DebugGoOn,"Channel '%s' already in list of '%s' driver [%p]",
+	    id().c_str(),m_driver->name().c_str(),this);
+	return;
+    }
+#endif
+    m_driver->m_total++;
+    m_driver->channels().append(this);
+    m_driver->changed();
+}
+
 void Channel::dropChan()
 {
     if (!m_driver)
@@ -398,7 +435,7 @@ void Channel::connected(const char* reason)
     Channel* peer = YOBJECT(Channel,getPeer());
     if (peer && peer->billid() && m_billid.null())
 	m_billid = peer->billid();
-    m_lastPeerId = getPeerId();
+    getPeerId(m_lastPeerId);
 }
 
 void Channel::disconnected(bool final, const char* reason)
@@ -441,7 +478,10 @@ Message* Channel::getDisconnect(const char* reason)
 void Channel::status(const char* newstat)
 {
     m_status = newstat;
-    m_answered = m_answered || (m_status == "answered");
+    if (!m_answered && (m_status == "answered")) {
+	m_maxcall = 0;
+	m_answered = true;
+    }
 }
 
 const char* Channel::direction() const
@@ -482,8 +522,9 @@ void Channel::complete(Message& msg, bool minimal) const
 	msg.setParam("targetid",m_targetid);
     if (m_billid)
 	msg.setParam("billid",m_billid);
-    if (getPeer())
-	msg.setParam("peerid",getPeer()->id());
+    String peer;
+    if (getPeerId(peer))
+	msg.setParam("peerid",peer);
     if (m_lastPeerId)
 	msg.setParam("lastpeerid",m_lastPeerId);
     msg.setParam("answered",String::boolText(m_answered));
@@ -626,6 +667,7 @@ void Channel::msgStatus(Message& msg)
 // Find a data endpoint to process it
 bool Channel::msgControl(Message& msg)
 {
+    setMaxcall(msg);
     for (ObjList* o = m_data.skipNull(); o; o = o->skipNext()) {
 	DataEndpoint* dep = static_cast<DataEndpoint*>(o->get());
 	if (dep->control(msg))
@@ -638,8 +680,9 @@ void Channel::statusParams(String& str)
 {
     if (m_driver)
 	str.append("module=",",") << m_driver->name();
-    if (getPeer())
-	str.append("peerid=",",") << getPeer()->id();
+    String peer;
+    if (getPeerId(peer))
+	str.append("peerid=",",") << peer;
     str.append("status=",",") << m_status;
     str << ",direction=" << direction();
     str << ",answered=" << m_answered;
