@@ -31,10 +31,10 @@ Configuration::Configuration()
 {
 }
 
-Configuration::Configuration(const char* filename)
+Configuration::Configuration(const char* filename, bool warn)
     : String(filename)
 {
-    load();
+    load(warn);
 }
 
 ObjList* Configuration::getSectHolder(const String& sect) const
@@ -153,7 +153,7 @@ void Configuration::setValue(const String& sect, const char* key, bool value)
     setValue(sect,key,String::boolText(value));
 }
 
-bool Configuration::load()
+bool Configuration::load(bool warn)
 {
     m_sections.clear();
     if (null())
@@ -190,15 +190,38 @@ bool Configuration::load()
 		continue;
 	    }
 	    int q = s.find('=');
-	    if (q > 0)
-		addValue(sect,s.substr(0,q).trimBlanks(),s.substr(q+1).trimBlanks());
+	    if (q <= 0)
+		continue;
+	    String key = s.substr(0,q).trimBlanks();
+	    if (key.null())
+		continue;
+	    s = s.substr(q+1);
+	    while (s.endsWith("\\",false)) {
+		// line continues onto next
+		s.assign(s,s.length()-1);
+		if (!::fgets(buf,sizeof(buf),f))
+		    break;
+		pc = ::strchr(buf,'\r');
+		if (pc)
+		    *pc = 0;
+		pc = ::strchr(buf,'\n');
+		if (pc)
+		    *pc = 0;
+		pc = buf;
+		while (*pc == ' ' || *pc == '\t')
+		    pc++;
+		s += pc;
+	    }
+	    addValue(sect,key,s.trimBlanks());
 	}
 	::fclose(f);
 	return true;
     }
-    int err = errno;
-    Debug(DebugNote,"Failed to open config file '%s', using defaults (%d: %s)",
-	c_str(),err,strerror(err));
+    if (warn) {
+	int err = errno;
+	Debug(DebugNote,"Failed to open config file '%s', using defaults (%d: %s)",
+	    c_str(),err,strerror(err));
+    }
     return false;
 }
 
@@ -221,10 +244,9 @@ bool Configuration::save() const
 	    for (unsigned int i = 0; i < n; i++) {
 		NamedString *ns = nl->getParam(i);
 		if (ns) {
-		    const char *v = ns->c_str();
-		    if (!v)
-			v = "";
-		    ::fprintf(f,"%s=%s\n",ns->name().c_str(),v);
+		    // add a space after a line that ends with backslash
+		    const char* bk = ns->endsWith("\\",false) ? " " : "";
+		    ::fprintf(f,"%s=%s%s\n",ns->name().safe(),ns->safe(),bk);
 		}
 	    }
 	}
