@@ -26,13 +26,13 @@
 #include <stdlib.h>
 
 extern "C" {
-#include "tables/all.h"
+#include "all.h"
 }
 
 using namespace TelEngine;
 
 
-static DataBlock s_empty;
+static const DataBlock s_empty;
 
 const DataBlock& DataBlock::empty()
 {
@@ -45,7 +45,8 @@ DataBlock::DataBlock()
 }
 
 DataBlock::DataBlock(const DataBlock& value)
-    : m_data(0), m_length(0)
+    : GenObject(),
+      m_data(0), m_length(0)
 {
     assign(value.data(),value.length());
 }
@@ -271,4 +272,96 @@ bool DataBlock::convert(const DataBlock& src, const String& sFormat,
     return true;
 }
 
+// Decode a single nibble, return -1 on error
+inline signed char hexDecode(char c)
+{
+    if (('0' <= c) && (c <= '9'))
+	return c - '0';
+    if (('A' <= c) && (c <= 'F'))
+	return c - 'A' + 10;
+    if (('a' <= c) && (c <= 'f'))
+	return c - 'a' + 10;
+    return -1;
+}
+
+// Build this data block from a hexadecimal string representation.
+// Each octet must be represented in the input string with 2 hexadecimal characters.
+// If a separator is specified, the octets in input string must be separated using
+//  exactly 1 separator. Only 1 leading or 1 trailing separators are allowed
+bool DataBlock::unHexify(const char* data, unsigned int len, char sep)
+{
+    clear();
+    if (!(data && len))
+	return true;
+
+    // Calculate the destination buffer length
+    unsigned int n = 0;
+    if (!sep) {
+	if (0 != (len % 2))
+	    return false;
+	n = len / 2;
+    }
+    else {
+	// Remove leading and trailing separators
+	if (data[0] == sep) {
+	    data++;
+	    len--;
+	}
+	if (len && data[len-1] == sep)
+	    len--;
+	// No more leading and trailing separators allowed
+	if (2 != (len % 3))
+	    return (bool)(len == 0);
+	n = (len + 1) / 3;
+    }
+    if (!n)
+	return true;
+
+    char* buf = (char*)::malloc(n);
+    unsigned int iBuf = 0;
+    for (unsigned int i = 0; i < len; i += (sep ? 3 : 2)) {
+	signed char c1 = hexDecode(data[i]);
+	signed char c2 = hexDecode(data[i+1]);
+	if (c1 == -1 || c2 == -1 || (sep && (iBuf != n - 1) && (sep != data[i+2])))
+	    break;
+	buf[iBuf++] = (c1 << 4) | c2;
+    }
+    if (iBuf >= n)
+	assign(buf,n,false);
+    else
+	::free(buf);
+    return (iBuf >= n);
+}
+
+String DataBlock::sqlEscape(char extraEsc) const
+{
+    unsigned int len = m_length;
+    unsigned int i;
+    for (i = 0; i < m_length; i++) {
+	char c = static_cast<char*>(m_data)[i];
+	if (c == '\0' || c == '\r' || c == '\n' || c == '\\' || c == '\'' || c == extraEsc)
+	    len++;
+    }
+    String tmp(' ',len);
+    char* d = const_cast<char*>(tmp.c_str());
+    for (i = 0; i < m_length; i++) {
+	char c = static_cast<char*>(m_data)[i];
+	if (c == '\0' || c == '\r' || c == '\n' || c == '\\' || c == '\'' || c == extraEsc)
+	    *d++ = '\\';
+	switch (c) {
+	    case '\0':
+		c = '0';
+		break;
+	    case '\r':
+		c = 'r';
+		break;
+	    case '\n':
+		c = 'n';
+		break;
+	}
+	*d++ = c;
+    }
+    return tmp;
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
