@@ -585,8 +585,9 @@ bool DataSource::attach(DataConsumer* consumer, bool override)
     Lock mylock(this);
     DataSource*& src = override ? consumer->m_override : consumer->m_source;
     if (src != this) {
-	if (src)
-	    src->detach(consumer);
+	RefPointer<DataSource> src2(src);
+	if (src2)
+	    src2->detach(consumer);
 	src = this;
     }
     consumer->synchronize(this);
@@ -1071,11 +1072,7 @@ bool ThreadedSource::running() const
 bool ThreadedSource::looping(bool runConsumers) const
 {
     Lock mylock(const_cast<ThreadedSource*>(this));
-    if (runConsumers) {
-	if (!(alive() && m_consumers.count()))
-	    return false;
-    }
-    else if (refcount() <= 1)
+    if ((refcount() <= 1) && !(runConsumers && alive() && m_consumers.count()))
 	return false;
     return m_thread && !m_thread->check(false) &&
 	m_thread->isCurrent() && !Engine::exiting();
