@@ -111,8 +111,12 @@ using namespace TelEngine;
 #ifndef CFG_PATH
 #define CFG_PATH "." PATH_SEP "conf.d"
 #endif
+#ifndef DLL_SUFFIX
 #define DLL_SUFFIX ".yate"
+#endif
+#ifndef CFG_SUFFIX
 #define CFG_SUFFIX ".conf"
+#endif
 
 
 // Supervisor control constants
@@ -140,8 +144,11 @@ static bool s_makeworker = true;
 static bool s_keepclosing = false;
 static bool s_nounload = false;
 static int s_super_handle = -1;
+static int s_run_attempt = 0;
+static bool s_interactive = true;
 static bool s_localsymbol = false;
 static bool s_logtruncate = false;
+static const char* s_logfile = 0;
 
 static void sighandler(int signal)
 {
@@ -157,6 +164,14 @@ static void sighandler(int signal)
 	    Engine::restart(0,false);
 	    break;
 	case SIGHUP:
+	    if (s_interactive) {
+		// console got closed so shutdown without writing to console
+		if (!s_logfile)
+		    Debugger::enableOutput(false);
+		Engine::halt(0);
+		break;
+	    }
+	    // intentionally fall through
 	case SIGQUIT:
 	    if (s_nextinit <= Time::now())
 		Engine::init();
@@ -207,7 +222,6 @@ static bool s_sigabrt = false;
 static bool s_lateabrt = false;
 static String s_cfgfile;
 static String s_userdir(CFG_DIR);
-static const char* s_logfile = 0;
 static Configuration s_cfg;
 static ObjList plugins;
 static ObjList* s_cmds = 0;
@@ -298,6 +312,7 @@ bool EngineStatusHandler::received(Message &msg)
     msg.retValue() << ",handlers=" << Engine::self()->handlerCount();
     msg.retValue() << ",messages=" << Engine::self()->messageCount();
     msg.retValue() << ",supervised=" << (s_super_handle >= 0);
+    msg.retValue() << ",runattempt=" << s_run_attempt;
     msg.retValue() << ",threads=" << Thread::count();
     msg.retValue() << ",workers=" << EnginePrivate::count;
     msg.retValue() << ",mutexes=" << Mutex::count();
@@ -650,10 +665,17 @@ static pid_t s_superpid = -1;
 static void superhandler(int signal)
 {
     switch (signal) {
-	case SIGHUP:
-	    if (s_logrotator)
-		s_rotatenow = true;
+	case SIGUSR1:
+	case SIGUSR2:
+	    s_rundelay = RUNDELAY_MIN;
 	    break;
+	case SIGHUP:
+	    if (!s_interactive) {
+		if (s_logrotator)
+		    s_rotatenow = true;
+		break;
+	    }
+	    // intentionally fall through
 	case SIGINT:
 	case SIGTERM:
 	case SIGABRT:
@@ -719,6 +741,7 @@ static int supervise(void)
 	while (::waitpid(-1,0,WNOHANG) > 0)
 	    ;
 	rotatelogs();
+	s_run_attempt++;
 	s_childpid = ::fork();
 	if (s_childpid < 0) {
 	    int err = errno;
@@ -843,7 +866,7 @@ SLib::SLib(HMODULE handle, const char* file, bool nounload)
 SLib::~SLib()
 {
 #ifdef DEBUG
-    Debugger debug("SLib::~SLib()"," [%p]",this);
+    Debugger debug("SLib::~SLib()"," '%s' [%p]",c_str(),this);
 #endif
     if (s_nounload || m_nounload) {
 #ifdef _WINDOWS
@@ -997,8 +1020,10 @@ int Engine::run()
     s_params.addParam("modulepath",s_modpath);
     s_params.addParam("modsuffix",s_modsuffix);
     s_params.addParam("logfile",s_logfile);
+    s_params.addParam("interactive",String::boolText(s_interactive));
     s_params.addParam("clientmode",String::boolText(clientMode()));
     s_params.addParam("supervised",String::boolText(s_super_handle >= 0));
+    s_params.addParam("runattempt",String(s_run_attempt));
     s_params.addParam("maxworkers",String(s_maxworkers));
     DDebug(DebugAll,"Engine::run()");
     install(new EngineStatusHandler);
@@ -1257,7 +1282,7 @@ bool Engine::loadPluginDir(const String& relPath)
 		loadPlugin(path + PATH_SEP + entry.cFileName,false,
 		    s_cfg.getBoolValue("nounload",entry.cFileName));
 	}
-    } while (::FindNextFile(hf,&entry));
+    } while (::FindNextFile(hf,&entry) && !exiting());
     ::FindClose(hf);
 #else
     DIR *dir = ::opendir(path);
@@ -1266,7 +1291,7 @@ bool Engine::loadPluginDir(const String& relPath)
 	return false;
     }
     struct dirent *entry;
-    while ((entry = ::readdir(dir)) != 0) {
+    while (((entry = ::readdir(dir)) != 0) && !exiting()) {
 	XDebug(DebugInfo,"Found dir entry %s",entry->d_name);
 	int n = ::strlen(entry->d_name) - s_modsuffix.length();
 	if ((n > 0) && !::strcmp(entry->d_name+n,s_modsuffix)) {
@@ -1290,6 +1315,8 @@ void Engine::loadPlugins()
             NamedString *n = l->getParam(i);
             if (n && n->toBoolean())
                 loadPlugin(n->name());
+	    if (exiting())
+		break;
 	}
     }
     loadPluginDir(String::empty());
@@ -1301,6 +1328,8 @@ void Engine::loadPlugins()
     if (l) {
         unsigned int len = l->length();
         for (unsigned int i=0; i<len; i++) {
+	    if (exiting())
+		return;
             NamedString *n = l->getParam(i);
             if (n && n->toBoolean())
                 loadPlugin(n->name());
@@ -1310,6 +1339,8 @@ void Engine::loadPlugins()
 
 void Engine::initPlugins()
 {
+    if (exiting())
+	return;
     Output("Initializing plugins");
     if (dispatch("engine.init"))
 	Debug(DebugGoOn,"Message engine.init was unexpectedly handled!");
@@ -1317,6 +1348,10 @@ void Engine::initPlugins()
     for (; l; l = l->skipNext()) {
 	Plugin *p = static_cast<Plugin *>(l->get());
 	p->initialize();
+	if (exiting()) {
+	    Output("Initialization aborted, exiting...");
+	    return;
+	}
     }
     Output("Initialization complete");
 }
@@ -1682,7 +1717,17 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 				    s_lateabrt = true;
 				    break;
 				case 'm':
-				    Lockable::wait(10000000);
+				    {
+					unsigned long lockWait = Lockable::wait();
+					if (lockWait) {
+					    lockWait /= 2;
+					    if (lockWait < Thread::idleUsec())
+						lockWait = Thread::idleUsec();
+					}
+					else
+					    lockWait = 10000000;
+					Lockable::wait(lockWait);
+				    }
 				    break;
 #ifdef RTLD_GLOBAL
 				case 'l':
@@ -1845,6 +1890,7 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 	    ::fprintf(stderr,"Daemonification failed: %s (%d)\n",::strerror(err),err);
 	    return err;
 	}
+	s_interactive = false;
     }
 #endif
 
@@ -1918,8 +1964,10 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
     Debugger::setFormatting(tstamp);
 
 #ifdef _WINDOWS
-    if (service)
+    if (service) {
+	s_interactive = false;
 	retcode = ::StartServiceCtrlDispatcher(dispatchTable) ? 0 : ::GetLastError();
+    }
     else
 #endif
 	retcode = engineRun();
