@@ -70,6 +70,8 @@ public:
 	{ return m_recursive; }
     inline const char* name() const
 	{ return m_name; }
+    inline const char* owner() const
+	{ return m_owner; }
     bool locked() const
     	{ return (m_locked > 0); }
     bool lock(long maxwait);
@@ -511,6 +513,10 @@ void Lockable::wait(unsigned long maxwait)
     s_maxwait = maxwait;
 }
 
+unsigned long Lockable::wait()
+{
+    return s_maxwait;
+}
 
 Mutex::Mutex(bool recursive, const char* name)
     : m_private(0)
@@ -570,6 +576,11 @@ bool Mutex::locked() const
     return m_private && m_private->locked();
 }
 
+const char* Mutex::owner() const
+{
+    return m_private ? m_private->owner() : static_cast<const char*>(0);
+}
+
 int Mutex::count()
 {
     return MutexPrivate::s_count;
