@@ -110,6 +110,9 @@ SignallingCircuitGroup* SignallingCallControl::attach(SignallingCircuitGroup* ci
 bool SignallingCallControl::reserveCircuit(SignallingCircuit*& cic, const char* range,
 	int checkLock, const String* list, bool mandatory, bool reverseRestrict)
 {
+    DDebug(DebugAll,"SignallingCallControl::reserveCircuit(%p,'%s',%d,'%s',%s,%s) [%p]",
+	cic,range,checkLock,TelEngine::c_safe(list),
+	String::boolText(mandatory),String::boolText(reverseRestrict),this);
     Lock mylock(this);
     releaseCircuit(cic);
     if (!m_circuits)
@@ -427,7 +430,7 @@ SignallingCircuit::SignallingCircuit(Type type, unsigned int code,
       m_group(group), m_span(span),
       m_code(code), m_type(type),
       m_status(Disabled),
-      m_lock(0), m_lastEvent(0)
+      m_lock(0), m_lastEvent(0), m_noEvents(true)
 {
     XDebug(m_group,DebugAll,"SignallingCircuit::SignallingCircuit [%p]",this);
 }
@@ -438,7 +441,7 @@ SignallingCircuit::SignallingCircuit(Type type, unsigned int code, Status status
       m_group(group), m_span(span),
       m_code(code), m_type(type),
       m_status(status),
-      m_lock(0), m_lastEvent(0)
+      m_lock(0), m_lastEvent(0), m_noEvents(true)
 {
     XDebug(m_group,DebugAll,"SignallingCircuit::SignallingCircuit [%p]",this);
 }
@@ -465,12 +468,16 @@ bool SignallingCircuit::setParams(const NamedList& params)
 // Get first event from queue
 SignallingCircuitEvent* SignallingCircuit::getEvent(const Time& when)
 {
+    if (m_noEvents)
+	return 0;
     Lock lock(m_mutex);
     if (m_lastEvent)
 	return 0;
     ObjList* obj = m_events.skipNull();
-    if (!obj)
+    if (!obj) {
+	m_noEvents = true;
 	return 0;
+    }
     m_lastEvent = static_cast<SignallingCircuitEvent*>(m_events.remove(obj->get(),false));
     return m_lastEvent;
 }
@@ -523,6 +530,7 @@ void SignallingCircuit::addEvent(SignallingCircuitEvent* event)
     if (!event)
 	return;
     Lock lock(m_mutex);
+    m_noEvents = false;
     m_events.append(event);
 }
 
@@ -885,6 +893,8 @@ unsigned int SignallingCircuitGroup::advance(unsigned int n, int strategy,
 SignallingCircuit* SignallingCircuitGroup::reserve(int checkLock, int strategy,
 	SignallingCircuitRange* range)
 {
+    DDebug(this,DebugInfo,"SignallingCircuitGroup::reserve(%d,%d,%p) [%p]",
+	checkLock,strategy,range,this);
     Lock mylock(this);
     if (!range)
 	range = &m_range;
@@ -959,6 +969,8 @@ SignallingCircuit* SignallingCircuitGroup::reserve(int checkLock, int strategy,
 SignallingCircuit* SignallingCircuitGroup::reserve(const String& list, bool mandatory,
 	int checkLock, int strategy, SignallingCircuitRange* range)
 {
+    DDebug(this,DebugInfo,"SignallingCircuitGroup::reserve('%s',%s,%d,%d,%p) [%p]",
+	list.c_str(),String::boolText(mandatory),checkLock,strategy,range,this);
     Lock mylock(this);
     if (!range)
 	range = &m_range;
