@@ -29,7 +29,7 @@
 using namespace TelEngine;
 namespace { // anonymous
 
-static Mutex s_mutex(true);
+static Mutex s_mutex(true,"CallGen");
 static ObjList s_calls;
 static Configuration s_cfg;
 static bool s_runs = false;
@@ -78,15 +78,15 @@ private:
 class DummyConsumer : public DataConsumer
 {
 public:
-    virtual void Consume(const DataBlock& data, unsigned long tStamp)
-	{ }
+    virtual unsigned long Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags)
+	{ return invalidStamp(); }
 };
 
 class GenThread : public Thread
 {
 public:
     GenThread()
-	: Thread("CallGen")
+	: Thread("CallGen Caller")
 	{ }
     virtual void run();
 };
@@ -95,7 +95,7 @@ class CleanThread : public Thread
 {
 public:
     CleanThread()
-	: Thread("GenCleaner")
+	: Thread("CallGen Cleaner")
 	{ }
     virtual void run();
 };
@@ -422,15 +422,30 @@ static const char* s_cmds[] = {
 
 bool CmdHandler::doComplete(const String& partLine, const String& partWord, String& rval)
 {
-    if (partLine.null() || (partLine == "help")) {
-	if (partWord.null() || String("callgen").startsWith(partWord))
-	    rval.append("callgen","\t");
-    }
+    if (partLine.null() && partWord.null())
+	return false;
+    if (partLine.null() || (partLine == "help"))
+	Module::itemComplete(rval,"callgen",partWord);
     else if (partLine == "callgen") {
-	for (const char** list = s_cmds; *list; list++) {
-	    String tmp = *list;
-	    if (partWord.null() || tmp.startsWith(partWord))
-		rval.append(tmp,"\t");
+	for (const char** list = s_cmds; *list; list++)
+	    Module::itemComplete(rval,*list,partWord);
+	return true;
+    }
+    else if (partLine == "callgen set") {
+	const NamedList* sect = s_cfg.getSection("parameters");
+	if (!sect)
+	    return false;
+	unsigned int n = sect->length();
+	for (unsigned int i = 0; i < n; i++) {
+	    const NamedString* param = sect->getParam(i);
+	    if (param) {
+		if (partWord == (param->name() + "=")) {
+		    // complete the value
+		    Module::itemComplete(rval,param->name() + "=" + *param,partWord);
+		    return true;
+		}
+		Module::itemComplete(rval,param->name(),partWord);
+	    }
 	}
 	return true;
     }
@@ -513,8 +528,11 @@ bool CmdHandler::doCommand(String& line, String& rval)
     }
     else if (line == "load") {
 	s_mutex.lock();
-	s_cfg.load();
-	rval << "Loaded config from " << s_cfg;
+	bool ok = s_cfg.load(false);
+	if (ok)
+	    rval << "Loaded config from " << s_cfg;
+	else
+	    rval << "Failed to load from " << s_cfg;
 	s_mutex.unlock();
     }
     else if (line == "save") {
@@ -609,8 +627,8 @@ void CallGenPlugin::initialize()
 {
     Output("Initializing module Call Generator");
     s_mutex.lock();
-    s_cfg = Engine::configFile("callgen");
-    s_cfg.load();
+    s_cfg = Engine::configFile("callgen",Engine::clientMode());
+    s_cfg.load(false);
     s_mutex.unlock();
     if (m_first) {
 	m_first = false;
