@@ -320,6 +320,8 @@ void ConfRoom::addChannel(ConfChan* chan, bool player)
 	Message* m = new Message("chan.notify");
 	m->addParam("id",chan->id());
 	m->addParam("targetid",m_notify);
+	if (chan->getPeerId())
+	    m->addParam("peerid",chan->getPeerId());
 	chan->populateMsg(*m);
 	m->addParam("event","joined");
 	m->addParam("maxusers",String(m_maxusers));
@@ -423,6 +425,7 @@ bool ConfRoom::setRecording(const NamedList& params)
     if (*record && (*record != "-") && record->toBoolean(true)) {
 	String warn = params.getValue("recordwarn");
 	ch = new ConfChan(this,!warn.null());
+	ch->initChan();
 	DDebug(&__plugin,DebugCall,"Starting record leg '%s' to '%s'",
 	    ch->id().c_str(),record->c_str());
 	Message* m = ch->message("call.execute");
@@ -852,6 +855,7 @@ bool ConfHandler::received(Message& msg)
 
     // create a conference leg or even a room for the caller
     ConfChan *c = new ConfChan(room,msg,counted,utility);
+    c->initChan();
     if (chan->connect(c,reason,false)) {
 	msg.setParam("peerid",c->id());
 	c->deref();
@@ -859,6 +863,7 @@ bool ConfHandler::received(Message& msg)
 	if (peer) {
 	    // create a conference leg for the old peer too
 	    ConfChan *p = new ConfChan(room,msg,counted,utility);
+	    p->initChan();
 	    peer->connect(p,reason,false);
 	    p->deref();
 	}
@@ -942,6 +947,7 @@ bool ConferenceDriver::msgExecute(Message& msg, String& dest)
     CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
     if (ch) {
 	ConfChan *c = new ConfChan(dest,msg,counted,utility);
+	c->initChan();
 	if (ch->connect(c)) {
 	    c->callConnect(msg);
 	    msg.setParam("peerid",c->id());
