@@ -57,7 +57,7 @@ class GsmCodec : public DataTranslator
 public:
     GsmCodec(const char* sFormat, const char* dFormat, bool encoding);
     ~GsmCodec();
-    virtual void Consume(const DataBlock& data, unsigned long tStamp);
+    virtual unsigned long Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags);
 private:
     bool m_encoding;
     gsm m_gsm;
@@ -84,10 +84,12 @@ GsmCodec::~GsmCodec()
     }
 }
 
-void GsmCodec::Consume(const DataBlock& data, unsigned long tStamp)
+unsigned long GsmCodec::Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags)
 {
     if (!(m_gsm && getTransSource()))
-	return;
+	return 0;
+    if (data.null() && (flags & DataSilent))
+	return getTransSource()->Forward(data,tStamp,flags);
     ref();
     m_data += data;
     DataBlock outdata;
@@ -120,11 +122,13 @@ void GsmCodec::Consume(const DataBlock& data, unsigned long tStamp)
     }
     XDebug("GsmCodec",DebugAll,"%scoding %d frames of %d input bytes (consumed %d) in %d output bytes",
 	m_encoding ? "en" : "de",frames,m_data.length(),consumed,outdata.length());
+    unsigned long len = 0;
     if (frames) {
 	m_data.cut(-consumed);
-	getTransSource()->Forward(outdata,tStamp);
+	len = getTransSource()->Forward(outdata,tStamp,flags);
     }
     deref();
+    return len;
 }
 
 GsmPlugin::GsmPlugin()
@@ -163,6 +167,13 @@ const TranslatorCaps* GsmPlugin::getCapabilities() const
 
 INIT_PLUGIN(GsmPlugin);
 
+UNLOAD_PLUGIN(unloadNow)
+{
+    if (unloadNow)
+	return !__plugin.isBusy();
+    return true;
+}
+
 }; // anonymous namespace
 
 /* vi: set ts=8 sw=4 sts=4 noet: */
