@@ -58,6 +58,7 @@ public:
 
 
 static PbxModule s_module;
+static const String s_masquerade("chan.masquerade");
 
 
 // Utility function to get a pointer to a call endpoint (or its peer) by id
@@ -111,19 +112,39 @@ bool ChanPickup::received(Message& msg)
     }
 
     // Connect parties and answer them
-    if (!called->connect(caller,msg.getValue("reason","pickup"))) {
+    const char* reason = msg.getValue("reason","pickup");
+    Debug(&s_module,DebugAll,"Channel '%s' picking up '%s' abandoning '%s', reason: '%s'",
+	caller->id().c_str(),called->id().c_str(),called->getPeerId().c_str(),reason);
+    if (!called->connect(caller,reason)) {
 	Debug(&s_module,DebugNote,"Pick up failed to connect '%s' to '%s'",
 	    caller->id().c_str(),called->id().c_str());
 	return false;
     }
+    msg.setParam("peerid",called->id());
+    msg.setParam("targetid",called->id());
 
-    Message* m = new Message("chan.masquerade");
-    m->addParam("id",caller->id());
-    m->addParam("message","call.answered");
-    Engine::enqueue(m);
-    m = new Message("chan.masquerade");
+    Message* m = new Message(s_masquerade);
     m->addParam("id",called->id());
     m->addParam("message","call.answered");
+    m->addParam("peerid",called->getPeerId());
+    m->addParam("reason",reason);
+    if (Engine::dispatch(m) || (s_masquerade != *m))
+	TelEngine::destruct(m);
+    else {
+	*m = "call.answered";
+	m->clearParam("message");
+	Engine::enqueue(m);
+    }
+    if (caller->getPeer() != called) {
+	Debug(&s_module,DebugMild,"Channel '%s' disconnected from '%s' while picking up",
+	    caller->id().c_str(),called->id().c_str());
+	return true;
+    }
+    m = new Message(s_masquerade);
+    m->addParam("id",caller->id());
+    m->addParam("message","call.answered");
+    m->addParam("peerid",called->getPeerId());
+    m->addParam("reason",reason);
     Engine::enqueue(m);
     return true;
 }
