@@ -690,6 +690,11 @@ SigChannel::SigChannel(SignallingEvent* event)
     plugin.copySigMsgParams(*m_route,event,&params);
     if (m_route->getBoolValue("overlapped") && !m_route->getValue("called"))
 	m_route->setParam("called","off-hook");
+    if (event->message()) {
+	const String* pres = event->message()->params().getParam("callerpres");
+	if (pres && (*pres == "restricted"))
+	    m_route->setParam("privacy",String::boolText(true));
+    }
 }
 
 // Construct an unstarted outgoing channel
@@ -805,7 +810,10 @@ bool SigChannel::startCall(Message& msg, SigTrunk* trunk)
     sigMsg->params().copyParam(msg,"format");
     sigMsg->params().copyParam(msg,"callernumtype");
     sigMsg->params().copyParam(msg,"callernumplan");
-    sigMsg->params().copyParam(msg,"callerpres");
+    if (msg.getValue("privacy") && msg.getBoolValue("privacy",true))
+	sigMsg->params().addParam("callerpres","restricted");
+    else
+	sigMsg->params().copyParam(msg,"callerpres");
     sigMsg->params().copyParam(msg,"callerscreening");
     sigMsg->params().copyParam(msg,"callednumtype");
     sigMsg->params().copyParam(msg,"callednumplan");
@@ -1220,6 +1228,8 @@ SignallingCircuitEvent* SigChannel::handleRtp(Message& msg)
     SignallingCircuit* cic = getCircuit();
     if (!cic)
 	return 0;
+    setSource();
+    setConsumer();
     SignallingCircuitEvent* ev = new SignallingCircuitEvent(cic,
 	SignallingCircuitEvent::Connect,"rtp");
     ev->copyParams(msg);
@@ -1354,10 +1364,12 @@ bool SigDriver::msgExecute(Message& msg, String& dest)
     // Create channel
     lock();
     SigChannel* sigCh = new SigChannel(msg.getValue("caller"),dest);
+    sigCh->initChan();
     unlock();
     bool ok = sigCh->startCall(msg,*trunk);
     if (ok) {
 	if (sigCh->connect(peer,msg.getValue("reason"))) {
+	    sigCh->callConnect(msg);
 	    msg.setParam("peerid",sigCh->id());
 	    msg.setParam("targetid",sigCh->id());
         }
@@ -1539,6 +1551,7 @@ void SigDriver::handleEvent(SignallingEvent* event)
     if (event->type() == SignallingEvent::NewCall) {
 	lock();
 	ch = new SigChannel(event);
+	ch->initChan();
 	unlock();
 	// Route the call
 	if (!ch->startRouter()) {
