@@ -43,7 +43,7 @@ namespace { // anonymous
 // each tone from threshold from total
 #define THRESHOLD2_REL_DTMF 0.33
 // hysteresis after tone detection
-#define THRESHOLD2_REL_HIST 0.90
+#define THRESHOLD2_REL_HIST 0.75
 
 // minimum DTMF detect time
 #define DETECT_DTMF_MSEC 32
@@ -96,7 +96,7 @@ public:
     };
     ToneConsumer(const String& id, const String& name);
     virtual ~ToneConsumer(); 
-    virtual void Consume(const DataBlock& data, unsigned long tStamp);
+    virtual unsigned long Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags);
     virtual const String& toString() const
 	{ return m_name; }
     inline const String& id() const
@@ -111,9 +111,12 @@ private:
     String m_divert;
     String m_caller;
     String m_called;
+    String m_target;
+    String m_dnis;
     Mode m_mode;
     bool m_detFax;
     bool m_detDtmf;
+    bool m_detDnis;
     char m_dtmfTone;
     int m_dtmfCount;
     double m_xv[3];
@@ -148,7 +151,7 @@ private:
     bool m_first;
 };
 
-static Mutex s_mutex;
+static Mutex s_mutex(false,"ToneDetect");
 static int s_count = 0;
 
 static ToneDetectorModule plugin;
@@ -205,7 +208,8 @@ void Tone2PoleFilter::update(double xd)
 
 
 ToneConsumer::ToneConsumer(const String& id, const String& name)
-    : m_id(id), m_name(name), m_mode(Mono), m_detFax(true), m_detDtmf(true),
+    : m_id(id), m_name(name), m_mode(Mono),
+      m_detFax(true), m_detDtmf(true), m_detDnis(false),
       m_fax(s_paramsCNG)
 { 
     Debug(&plugin,DebugAll,"ToneConsumer::ToneConsumer(%s,'%s') [%p]",
@@ -228,7 +232,7 @@ ToneConsumer::ToneConsumer(const String& id, const String& name)
 	m_format = "2*slin";
     if (tmp && (tmp != "*")) {
 	// individual detection requested
-	m_detFax = m_detDtmf = false;
+	m_detFax = m_detDtmf = m_detDnis = false;
 	ObjList* k = tmp.split(',',false);
 	for (ObjList* l = k; l; l = l->next()) {
 	    String* s = static_cast<String*>(l->get());
@@ -241,6 +245,10 @@ ToneConsumer::ToneConsumer(const String& id, const String& name)
 		m_fax.assign(s_paramsCED);
 		m_detFax = true;
 	    }
+	    else if (*s == "callsetup") {
+		// call setup info in the form *ANI*DNIS*
+		m_detDnis = true;
+	    }
 	}
 	TelEngine::destruct(k);
     }
@@ -294,15 +302,17 @@ void ToneConsumer::checkDtmf()
 	}
     }
     double limitAll = m_pwr*THRESHOLD2_REL_ALL;
-    if (c)
-	limitAll *= THRESHOLD2_REL_HIST;
     double limitOne = limitAll*THRESHOLD2_REL_DTMF;
+    if (c) {
+	limitAll *= THRESHOLD2_REL_HIST;
+	limitOne *= THRESHOLD2_REL_HIST;
+    }
     if ((maxL < limitOne) ||
 	(maxH < limitOne) ||
 	((maxL+maxH) < limitAll)) {
 #ifdef DEBUG
 	if (c)
-	    Debug(&plugin,DebugAll,"Giving up DTMF '%c' lo=%f, hi=%f, total=%f",
+	    Debug(&plugin,DebugInfo,"Giving up DTMF '%c' lo=%0.1f, hi=%0.1f, total=%0.1f",
 		c,maxL,maxH,m_pwr);
 #endif
 	return;
@@ -311,22 +321,40 @@ void ToneConsumer::checkDtmf()
     buf[0] = s_tableDtmf[l][h];
     buf[1] = '\0';
     if (buf[0] != c) {
-	DDebug(&plugin,DebugAll,"DTMF '%s' new candidate on %s, lo=%f, hi=%f, total=%f",
+	DDebug(&plugin,DebugInfo,"DTMF '%s' new candidate on %s, lo=%0.1f, hi=%0.1f, total=%0.1f",
 	    buf,m_id.c_str(),maxL,maxH,m_pwr);
 	m_dtmfTone = buf[0];
 	m_dtmfCount = 1;
 	return;
     }
     m_dtmfTone = c;
-    XDebug(&plugin,DebugInfo,"DTMF '%s' candidate %d on %s, lo=%f, hi=%f, total=%f",
+    XDebug(&plugin,DebugAll,"DTMF '%s' candidate %d on %s, lo=%0.1f, hi=%0.1f, total=%0.1f",
 	buf,m_dtmfCount,m_id.c_str(),maxL,maxH,m_pwr);
     if (m_dtmfCount++ == DETECT_DTMF_MSEC) {
-	DDebug(&plugin,DebugInfo,"DTMF '%s' detected on %s, lo=%f, hi=%f, total=%f",
+	DDebug(&plugin,DebugNote,"%sDTMF '%s' detected on %s, lo=%0.1f, hi=%0.1f, total=%0.1f",
+	    (m_detDnis ? "DNIS/" : ""),
 	    buf,m_id.c_str(),maxL,maxH,m_pwr);
+	if (m_detDnis) {
+	    static Regexp r("^\\*\\([0-9#]*\\)\\*\\([0-9#]*\\)\\*$");
+	    m_dnis += buf;
+	    if (m_dnis.matches(r)) {
+		m_detDnis = false;
+		Message* m = new Message("chan.notify");
+		m->addParam("id",m_id);
+		if (m_target)
+		    m->addParam("targetid",m_target);
+		m->addParam("operation","setup");
+		m->addParam("caller",m_dnis.matchString(1));
+		m->addParam("called",m_dnis.matchString(2));
+		Engine::enqueue(m);
+	    }
+	    return;
+	}
 	Message *m = new Message("chan.masquerade");
 	m->addParam("id",m_id);
 	m->addParam("message","chan.dtmf");
 	m->addParam("text",buf);
+	m->addParam("detected","inband");
 	Engine::enqueue(m);
     }
 }
@@ -337,12 +365,12 @@ void ToneConsumer::checkFax()
     if (m_fax.value() < m_pwr*THRESHOLD2_REL_FAX)
 	return;
     if (m_fax.value() > m_pwr) {
-	DDebug(&plugin,DebugNote,"Overshoot on %s, signal=%f, total=%f",
+	DDebug(&plugin,DebugNote,"Overshoot on %s, signal=%0.2f, total=%0.2f",
 	    m_id.c_str(),m_fax.value(),m_pwr);
 	init();
 	return;
     }
-    DDebug(&plugin,DebugInfo,"Fax detected on %s, signal=%f, total=%f",
+    DDebug(&plugin,DebugInfo,"Fax detected on %s, signal=%0.1f, total=%0.1f",
 	m_id.c_str(),m_fax.value(),m_pwr);
     // prepare for new detection
     init();
@@ -356,22 +384,24 @@ void ToneConsumer::checkFax()
 	m->addParam("callto",m_divert);
 	m->addParam("reason","fax");
     }
-    else
+    else {
 	m->addParam("message","call.fax");
+	m->addParam("detected","inband");
+    }
     Engine::enqueue(m);
 }
 
 // Feed samples to the filter(s)
-void ToneConsumer::Consume(const DataBlock& data, unsigned long timeDelta)
+unsigned long ToneConsumer::Consume(const DataBlock& data, unsigned long tStamp, unsigned long flags)
 {
     unsigned int samp = data.length() / 2;
     if (m_mode != Mono)
 	samp /= 2;
     if (!samp)
-	return;
+	return 0;
     const int16_t* s = (const int16_t*)data.data();
     if (!s)
-	return;
+	return 0;
     while (samp--) {
 	m_xv[0] = m_xv[1]; m_xv[1] = m_xv[2];
 	switch (m_mode) {
@@ -399,7 +429,7 @@ void ToneConsumer::Consume(const DataBlock& data, unsigned long timeDelta)
 	// update all active detectors
 	if (m_detFax)
 	    m_fax.update(dx);
-	if (m_detDtmf) {
+	if (m_detDtmf || m_detDnis) {
 	    for (int j = 0; j < 4; j++) {
 		m_dtmfL[j].update(dx);
 		m_dtmfH[j].update(dx);
@@ -410,7 +440,7 @@ void ToneConsumer::Consume(const DataBlock& data, unsigned long timeDelta)
 	    continue;
 	// is it enough total power to accept a signal?
 	if (m_pwr >= THRESHOLD2_ABS) {
-	    if (m_detDtmf)
+	    if (m_detDtmf || m_detDnis)
 		checkDtmf();
 	    if (m_detFax)
 		checkFax();
@@ -420,13 +450,17 @@ void ToneConsumer::Consume(const DataBlock& data, unsigned long timeDelta)
 	    m_dtmfCount = 0;
 	}
     }
-    XDebug(&plugin,DebugInfo,"Fax detector on %s: signal=%f, total=%f",
+    XDebug(&plugin,DebugAll,"Fax detector on %s: signal=%0.1f, total=%0.1f",
 	m_id.c_str(),m_fax.value(),m_pwr);
+    return invalidStamp();
 }
 
 // Copy parameters required for automatic fax call diversion
 void ToneConsumer::setDivert(const Message& msg)
 {
+    m_target = msg.getParam("notify");
+    if (m_id.null())
+	m_id = m_target;
     NamedString* divert = msg.getParam("fax_divert");
     if (!divert)
 	return;
@@ -454,6 +488,7 @@ bool AttachHandler::received(Message& msg)
     if (cons.null() && snif.null())
 	return false;
     CallEndpoint* ch = static_cast<CallEndpoint *>(msg.userObject("CallEndpoint"));
+    DataSource* ds = static_cast<DataSource *>(msg.userObject("DataSource"));
     if (ch) {
 	if (cons) {
 	    ToneConsumer* c = new ToneConsumer(ch->id(),cons);
@@ -478,6 +513,17 @@ bool AttachHandler::received(Message& msg)
 	}
 	return msg.getBoolValue("single");
     }
+    else if (ds && cons) {
+	ToneConsumer* c = new ToneConsumer(msg.getValue("id"),cons);
+	c->setDivert(msg);
+	bool ok = DataTranslator::attachChain(ds,c);
+	if (ok)
+	    msg.userData(c);
+	else
+	    msg.setParam("reason","attach-failure");
+	c->deref();
+	return ok && msg.getBoolValue("single");
+    }
     else
 	Debug(&plugin,DebugWarn,"ToneDetector attach request with no call endpoint!");
     return false;
