@@ -35,7 +35,7 @@ using namespace TelEngine;
 namespace { // anonymous
 
 /*
-  chan.stun parameters
+  socket.stun parameters
 
   uselocalusername        Add USERNAME attribute when sending requests
                           Defaults to true
@@ -64,7 +64,7 @@ class YStunMessage;                      // STUN message
 class YStunUtils;                        // General usefull functions
 class YStunMessageOut;                   // Outgoing STUN message (message + retransmission info)
 class YStunSocketFilter;                 // Socket filter for STUN
-class StunHandler;                       // chan.stun handler
+class StunHandler;                       // socket.stun handler
 class YStunPlugin;                       // The plugin
 
 /**
@@ -100,9 +100,9 @@ class YStunPlugin;                       // The plugin
 inline void setHeader(u_int8_t* buffer, u_int16_t type, u_int16_t len)
 {
     buffer[0] = type >> 8;
-    buffer[1] = type;
+    buffer[1] = (u_int8_t)type;
     buffer[2] = len >> 8;
-    buffer[3] = len;
+    buffer[3] = (u_int8_t)len;
 }
 
 // Get message header values (type+length)
@@ -329,7 +329,7 @@ public:
     static bool getAttrAuth(YStunMessage* msg, YStunAttribute::Type type,
 	String& auth);
 protected:
-    static Mutex m_idMutex;              // Lock id changes
+    static Mutex s_idMutex;              // Lock id changes
     static unsigned int m_id;            // Used to generate unique id for requests
 };
 
@@ -416,12 +416,12 @@ private:
 };
 
 /**
- * chan.stun message handler
+ * socket.stun message handler
  */
 class StunHandler : public MessageHandler
 {
 public:
-    StunHandler() : MessageHandler("chan.stun") {}
+    StunHandler() : MessageHandler("socket.stun") {}
     // Process message. Create and install filter.
     virtual bool received(Message &msg);
 };
@@ -573,7 +573,7 @@ bool YStunAttributeAddr::fromBuffer(u_int8_t* buffer, u_int16_t len)
 void YStunAttributeAddr::toBuffer(DataBlock& buffer)
 {
     u_int8_t header[12] = {0,0,0,0,
-		0,STUN_ATTR_ADDR_IPV4,m_port >> 8,m_port,
+		0,STUN_ATTR_ADDR_IPV4,m_port >> 8,(u_int8_t)m_port,
 		0,0,0,0};
     setHeader(header,type(),8);
     for (int start = 0, i = 8; i < 12; i++) {
@@ -701,7 +701,7 @@ void YStunMessage::print()
  * YStunUtils
  */
 unsigned int YStunUtils::m_id = 1;
-Mutex YStunUtils::m_idMutex(true);
+Mutex YStunUtils::s_idMutex(true,"YStunUtils::id");
 
 YStunUtils::YStunUtils()
 {
@@ -815,9 +815,9 @@ YStunMessage* YStunUtils::decode(const void* data, u_int32_t len, bool& isStun)
 void YStunUtils::createId(String& id)
 {
     id = "";
-    m_idMutex.lock();
+    s_idMutex.lock();
     id << m_id++ << "_";
-    m_idMutex.unlock();
+    s_idMutex.unlock();
     for (; id.length() < STUN_MSG_IDLENGTH;)
  	id << (int)random();
     id = id.substr(0,STUN_MSG_IDLENGTH);
@@ -838,9 +838,11 @@ bool YStunUtils::sendMessage(Socket* socket, const YStunMessage* msg,
     if (!socket->canRetry())
 	Debug(&iplugin,DebugWarn,"Socket write error: '%s' (%d). [%p]",
 	    ::strerror(socket->error()),socket->error(),sender);
+#ifdef DEBUG
     else
-	DDebug(&iplugin,DebugMild,"Socket temporary unavailable: '%s' (%d). [%p]",
+	Debug(&iplugin,DebugMild,"Socket temporary unavailable: '%s' (%d). [%p]",
 	    ::strerror(socket->error()),socket->error(),sender);
+#endif
     return false;
 }
 
@@ -936,7 +938,7 @@ YStunSocketFilter::YStunSocketFilter()
       m_useLocalUsername(false),
       m_useRemoteUsername(false),
       m_bindReq(0),
-      m_bindReqMutex(true),
+      m_bindReqMutex(true,"YStunSocketFilter::bindReq"),
       m_bindReqNext(0),
       m_notFound(true)
 {
