@@ -1128,12 +1128,18 @@ public:
     inline CallEndpoint* getPeer() const
 	{ return m_peer; }
 
+    /**
+     * Get the connected peer call id in a caller supplied String
+     * @param id String to fill in
+     * @return True if the call endpoint had a peer
+     */
+    bool getPeerId(String& id) const;
+
     /**
      * Get the connected peer call id
      * @return Connected peer call id or empty string
      */
-    inline const String& getPeerId() const
-	{ return m_peer ? m_peer->id() : String::empty(); }
+    String getPeerId() const;
 
     /**
      * Get the mutex that serializes access to this call endpoint, if any
@@ -1893,6 +1899,12 @@ public:
     const String& lastPeerId() const
 	{ return m_lastPeerId; }
 
+    /**
+     * Add the channel to the parent driver list
+     * This method must be called exactly once after the object is fully constructed
+     */
+    void initChan();
+
     /**
      * Start a routing thread for this channel, dereference dynamic channels
      * @param msg Pointer to message to route, typically a "call.route", will be
